import pg from 'pg'
import { sendProjectStatus } from './log'

export const PgPort = process.env.NODE_ENV?.trim() === 'production' ? 25556 : 30001
export const PgHost = process.env.NODE_ENV?.trim() === 'production' ? '192.168.0.4' : process.env.DEVHOST

let pool: pg.Pool = new pg.Pool({
  host: PgHost,
  port: PgPort,
  database: 'backend_helper'
})

export async function createPgClient (payload: {
  user?: string
  password?: string
  database?: string
  host?: string
  port?: number
}) {
  let client = new pg.Client({
    user: payload.user,
    password: payload.password,
    database: payload.database,
    host: payload.host || PgHost,
    port: payload.port || PgPort
  })
  await client.connect()
  return client
}

export function executePgQuery ({ client, query }: { client: pg.Client, query: string }) {
  return client.query(query)
}
export function createPgClientFactory (database?: string) {
  return createPgClient({ database })
}
export async function stopPgClient (pgClient: pg.Client) {
  await pgClient.end()
}

export type TableUser = { username: string, port: number }

export type TableV2 = {
  username: string
  status: 'null' | 'created' | 'stopped' | 'running' | 'launched' | 'creating' | 'compiling' | 'compiled'
  tenants: Record<string, any>
  debug: Record<string, any>
  has_debugged: boolean
  serverProperties: { id: any, key: string, value: string }[]
}

export type TableV1 = {
  username: string
  status:  'null' | 'created' | 'stopped' | 'running' | 'launched'  | 'creating' | 'compiling' | 'compiled'
  serverProperties: { id: any, key: string, value: string }[]
}

export async function getPgTableData (table: 'v1', username: string): Promise<TableV1 | undefined>;
export async function getPgTableData (table: 'v2', username: string): Promise<TableV2 | undefined>;
export async function getPgTableData (table: 'user', username: string): Promise<TableUser | undefined>;
export async function getPgTableData (table: string, username: string) {
  const result = await pool.query(`select * from public.${table} where username = '${username}'`)
  return result.rows[0]
}

export async function setPgTableData (table: 'user', username: string, key: keyof TableUser, value: TableUser[keyof TableUser]): Promise<void>;
export async function setPgTableData (table: 'v2', username: string, key: keyof TableV2, value: TableV2[keyof TableV2]): Promise<void>;
export async function setPgTableData (table: 'v1', username: string, key: keyof TableV1, value: TableV1[keyof TableV1]): Promise<void>;
export async function setPgTableData (table: 'v1' | 'v2' | 'user', username: string, key: string, value: any) {
  await pool.query(`UPDATE public.${table} SET "${key}"='${value}' WHERE username='${username}'`)
}

export async function insertPgTableData (table: 'v1', username: string): Promise<void>;
export async function insertPgTableData (table: 'v2', username: string): Promise<undefined>;
export async function insertPgTableData (table: 'user', username: string): Promise<undefined>;
export async function insertPgTableData (table: string, username: string) {
  await pool.query(`INSERT INTO public."${table}"(username) VALUES ('${username}');`)
}

export async function changeProjectStatus (table: 'v2', username: string, key: keyof TableV2, value: TableV2[keyof TableV2]): Promise<void>;
export async function changeProjectStatus (table: 'v1', username: string, key: keyof TableV1, value: TableV1[keyof TableV1]): Promise<void>;
export async function changeProjectStatus (table: string, username: string, key: string, value: any) {
  await setPgTableData(table as any, username, key as any, value)
  sendProjectStatus(table as any, username, value)
}

